'''
Created on Apr 13, 2011

@author: Panagiotis

Usage: clusters.py [options] <bids>

Options
    --help, -h           help
    --levels=, -l        levels of hierarchical clustering  (default: 2) 
    --output=, -o        output prefix
  
'''

import sys
import getopt
import logging
from collections import defaultdict
from itertools import combinations

from clustering.spectral import bipartite_graph_hierachical_clusters


def main(argv):
  if argv is None:
    argv = sys.argv
  try:
    opts, args = getopt.getopt(argv[1:], 'hl:o:', ['help', 'levels=', 'output='])
  except getopt.GetoptError as e:
    print str(e)
    print __doc__
    return 1
  if len(args) != 1:
    print __doc__
    return 1
  bids_fn = args[0]
  levels = 2
  output_prefix = None
  for o, v in opts:
    if o in ['--help', '-h']:
      print __doc__
      return 0
    elif o in ['--levels', '-l']:
      levels = int(v)
    elif o in ['--output', '-o']:
      output_prefix = v
  logging.basicConfig(format="%(asctime)s\t%(levelname)s\t%(message)s", level=logging.DEBUG)

  bids = read_bids_list(bids_fn)
  level = 1
  for advertiser2cluster, phrase2cluster in bipartite_graph_hierachical_clusters(bids, levels):
    print_clusters(advertiser2cluster, phrase2cluster, level, output_prefix)
    level += 1


def print_clusters(advertiser2cluster, phrase2cluster, level=0, output_prefix=None):
  # Print phrase clusters
  output_fn = None if output_prefix == None \
    else '%sphrases.%d.txt' % (output_prefix, level)
  output_f = sys.stdout if output_fn is None else open(output_fn, 'w')
  for phrase, cluster in phrase2cluster.iteritems():
    output_f.write('%d\t%d\n' % (phrase, cluster))
  if output_f != sys.stdout:
    output_f.close()
  # Print advertiser clusters
  output_fn = None if output_prefix == None \
    else '%sadvertisers.%d.txt' % (output_prefix, level)
  output_f = sys.stdout if output_fn is None else open(output_fn, 'w')
  for advertiser, cluster in advertiser2cluster.iteritems():
    output_f.write('%d\t%d\n' % (advertiser, cluster))
  if output_f != sys.stdout:
    output_f.close()


def read_bids_list(bids_fn):
  bids_f = open(bids_fn)
  bids_f.readline()                      # skip header
  bids = []
  for line in bids_f:
    ws = line.split()
    advertiser = int(ws[0])
    phrase = int(ws[1])
    bid = float(ws[2])
    bids.append((advertiser, phrase, bid))
  bids_f.close()
  return bids
      
if __name__ == '__main__':
  sys.exit(main(sys.argv))
