'''
Created on Apr 13, 2011

@author: Panagiotis

Usage: generate_breeding_advertisers.py [options] <bids> <k> <p>

Options
    --help, -h           help
    --seed, -s           seed for random number generator
  
'''

import sys
import getopt
import random
import heapq
from collections import defaultdict


from io.util import read_int_dict


def main(argv):
  if argv is None:
    argv = sys.argv
  try:
    opts, args = getopt.getopt(argv[1:], 'hs:', ['help', 'seed='])
  except getopt.GetoptError as e:
    print str(e)
    print __doc__
    return 1
  if len(args) != 3:
    print __doc__
    return 1
  bids_fn = args[0]
  k = int(args[1])
  p = float(args[2])
  seed = None
  
  for o, v in opts:
    if o in ['--help', '-h']:
      print __doc__
      return 0
    elif o in ['--seed', '-s']:
      seed = int(v)
  
  advertiser2bid = read_advertiser_bids(bids_fn)
  conflict_advertisers = set(weighted_sample(advertiser2bid, k, seed))
  for advertiser in advertiser2bid:
    if advertiser in conflict_advertisers:
      continue
    for conflict_advertiser in conflict_advertisers:
      if random.random() <= p:
        print '%d\t%d' % (advertiser, conflict_advertiser)
    

def weighted_sample(item2weight, k, seed=None):
  random.seed(seed)
  topk_items = []
  for item, weight in item2weight.iteritems():
    if weight == 0:
      continue
    selection_weight = random.random() ** (1 / weight) # corresponds to algorithm k_i
    if len(topk_items) < k:
      heapq.heappush(topk_items, (selection_weight, item))
    else:
      if selection_weight > topk_items[0][0]:
        heapq.heappushpop(topk_items, (selection_weight, item))
  return [item for (selection_weight, item) in topk_items]


def read_advertiser_bids(bids_fn):
  bids_f = open(bids_fn)
  bids_f.readline()                      # skip header
  advertiser2sum = defaultdict(float)
  for line in bids_f:
    ws = line.split()
    advertiser = int(ws[0])
    bid = float(ws[2])
    advertiser2sum[advertiser] += bid
  bids_f.close()
  return advertiser2sum
    
if __name__ == '__main__':
  sys.exit(main(sys.argv))