'''
Created on Apr 13, 2011

@author: Panagiotis

Usage: similar_bidders.py [options] <bids>

Options
    --help, -h           help
  
'''

import sys
import getopt
from collections import defaultdict
from itertools import combinations


def main(argv):
  if argv is None:
    argv = sys.argv
  try:
    opts, args = getopt.getopt(argv[1:], 'h', ['help'])
  except getopt.GetoptError as e:
    print str(e)
    print __doc__
    return 1
  if len(args) != 1:
    print __doc__
    return 1
  bids_fn = args[0]
  
  for o, v in opts:
    if o in ['--help', '-h']:
      print __doc__
      return 0
  advertiser2keywords = read_bids(bids_fn)
  
  for ai, aj, sim, support in similar_advertisers(advertiser2keywords, thres=0.1):
    print "%d\t%d\t%.2f\t%d" % (ai, aj, sim, support)


def read_bids(bids_fn):
  bids_f = open(bids_fn)
  bids_f.readline()                      # skip header
  advertiser2keywords = defaultdict(set)
  for line in bids_f:
    ws = map(int, line.split()[0:2])
    advertiser2keywords[ws[0]].add(ws[1])
  bids_f.close()
  return advertiser2keywords 

    
def similar_advertisers(advertiser2keywords, thres=0.5, min_support=1):
  for ai, aj in combinations(advertiser2keywords.iterkeys(), 2):
    keywordsi = advertiser2keywords[ai]
    keywordsj = advertiser2keywords[aj]
    support = len(keywordsi & keywordsj)
    if support < min_support:
      continue
    sim = jaccard(advertiser2keywords[ai], advertiser2keywords[aj])
    if sim >= thres:
      yield (ai, aj, sim, support)

def jaccard(a, b):
  if len(a) == 0 or len(b) == 0:
    return 0
  return 1.0 * len(a & b) / len(a | b)
      
if __name__ == '__main__':
  sys.exit(main(sys.argv))
           
