'''
Created on Jul 25, 2011

@author: Panagiotis
'''

import logging

from operator import itemgetter
from datetime import datetime

from allocator import Allocator

class Debbassac(Allocator):
  
  def __init__(self, advertiser2bid, advertiser2conflicts, rhos, keyword=None):
    Allocator.__init__(self, advertiser2bid, advertiser2conflicts, rhos, keyword)
  
  def allocate_slots(self, A=None, to_price=False):
    self.x_star = []
    self.w_star = 0
    if A is None:
      A = self.sorted_advertisers
    t_start = datetime.now()
    self.dfbnb([], 0, A)
    t_price_start = datetime.now()
    if to_price:
      self.p_star = self.determine_prices()
    t_end = datetime.now()
    self.delta_winners = (t_price_start - t_start).total_seconds()
    self.delta_price = (t_end - t_price_start).total_seconds()
    
    
  def determine_prices(self):
    # Save global solutions
    x_star = self.x_star
    w_star = self.w_star
    p_star = []
    for i, (a, v) in enumerate(x_star):
      self.allocate_slots(filter(lambda x : x[0] != a, self.sorted_advertisers))
      price = (self.w_star - (w_star - v * self.rhos[i])) / self.rhos[i]
      p_star.append(price)
    # Recover optimal allocation and welfare
    self.x_star = x_star
    self.w_star = w_star
    return p_star
  
  # @rem: remaining advertisers sorted by click valuation
  def dfbnb(self, x_in, w_in, rem):
    i = len(x_in)
    # Recursion termination
    if i == self.m or len(rem) == 0:
      if w_in > self.w_star:
        self.w_star = w_in
        self.x_star = x_in[:]
      return
    # Calculate current welfare and future welfare heuristic
    ws = []
    fs = []  # f = w + h 
    rems = []
    last_value = None
    last_index = 0
    for index, (a, v) in enumerate(rem):
      # Update last index
      if v != last_value:
        last_value = v
        last_index = index
      #
      w_a =  w_in + self.rhos[i] * v
      rem_a = self.filter_remaining_advertisers(a, rem[last_index:])
      h_a = sum([x * y for x, y in 
                 zip(map(itemgetter(1), rem_a[:(self.m - (i + 1))]), 
                     self.rhos[(i+1):])])
      ws.append(w_a)
      fs.append(w_a + h_a)
      rems.append(rem_a)
      
      if len(self.advertiser2conflicts[a]) == 0:
        break

    # Explore tree
    for j in sorted(range(len(rem[:(index+1)])), cmp=lambda j, k: cmp(fs[j], fs[k]), reverse=True):
      if fs[j] <= self.w_star:
        break
      else:
        self.dfbnb(x_in + [rem[j]], ws[j], rems[j])
      

  def filter_remaining_advertisers(self, a, rem):
    value = self.advertiser2bid[a]
    conflicts = self.advertiser2conflicts.get(a, set())
    filtered = []
    for aj, vj in rem:
      if vj > value:
        logging.warn("Must not be here...")
      elif aj in conflicts or aj == a:
        continue
      else:
        filtered.append((aj, vj))
    return filtered  
  
# Test
if __name__ == '__main__':
  logging.basicConfig(format="%(asctime)s\t%(levelname)s\t%(message)s", level=logging.DEBUG)
  advertiser2bid = {1:10, 2:8, 3:5, 4:2}
  advertiser2conflicts = {1:set([2,3])}
  allocator = Debbassac(advertiser2bid, advertiser2conflicts, [0.5, 0.25])
  allocator.allocate_slots()
  print allocator.x_star, allocator.w_star